/*
 * Iparticle.h
 *
 *      Author: kappl
 */

#ifndef IPARTICLE_H_
#define IPARTICLE_H_

#include "properties.h"
#include "stringconst.h"
#include "random.h"

/**
 * @brief Abstract base class (interface) for Particle
 */
class IParticle
{
	protected:
	virtual void step(Random&) = 0;

	virtual	long getDeltaTime() const = 0;
	virtual	double getDeltaT() const = 0;
	virtual double getDeltaR(Random&) const = 0;
	virtual double getDeltaTheta(Random&) const = 0;
	virtual	double getDeltaPhi() const = 0;

	virtual void calculate() = 0;

	virtual double getTheta() const = 0;
	virtual double getPhi() const = 0;
	virtual double getTime() const = 0;
	virtual double getMass() const = 0;
	virtual double getCharge() const = 0;
	virtual double getV() const = 0;
	virtual double getP2() const = 0;
	virtual double getRigidity() const = 0;
	virtual double getBeta() const = 0;
	virtual double getPolarity() const = 0;
	virtual double getAngle() const = 0;
	virtual double getTanPsi() const = 0;
	virtual double convertDeg(double) const = 0;
	virtual double getLarmorRadius() const = 0;
	virtual double getB0OverB() const = 0;
	virtual double getB() const = 0;
	virtual int getHeaviside() const = 0;
	virtual double getKappaPerpendicular() const = 0;
	virtual double getKappaThetaTheta() const = 0;

	public:
	virtual ~IParticle() {};

	virtual double getHeliosphereBoundary() const = 0;
	virtual long simulate(Random&) = 0;
	virtual double getR() const = 0;
	virtual double getT() const = 0;

	virtual double getISFlux(int) const = 0;
	virtual double getISFlux(bool) const = 0;
	virtual double getISFlux(bool, int) const = 0;
};

#endif /* IPARTICLE_H_ */
