/*
 * distribution.cc
 *
 *      Author: kappl
 */

#include "distribution.h"
#include "constants.h"
#include <iostream>
#include <vector>
#include <string>

/**
 * @brief Constructor to build distribution with input data from a density
 *
 * @param den Density
 * @param ekin Vector of kinetic energies which should be used for the distribution bins
 */
Distribution::Distribution(Density &den, const std::vector<double> &ekin) {
	int red = 0;

	for (unsigned int j = 0; j < ekin.size(); j++)
	{
		// set initial flux to zero
		flux.push_back(0);
	}

	for (int i= 0; i < den.getTotalNumber(); i++)
	{
		if (den.getParticle(i)->getR() > den.getParticle(i)->getHeliosphereBoundary())
		{
			for (unsigned int j = 0; j < ekin.size(); j++)
			{
				if (j == 0)
				{
					if ((den.getParticle(i)->getT() >= ekin[j]) &&
								(den.getParticle(i)->getT() < ekin[j] + (ekin[j+1] - ekin[j])/2.))
					{
						flux[j]++;
					}
				}
				else if (j == ekin.size() - 1)
				{
					if ((den.getParticle(i)->getT() >= ekin[j-1] + (ekin[j] - ekin[j-1])/2.) &&
							(den.getParticle(i)->getT() < ekin[j]))
					{
						flux[j]++;
					}
				}
				else
				{
					if ((den.getParticle(i)->getT() >= ekin[j] - (ekin[j] - ekin[j-1])/2.) &&
							(den.getParticle(i)->getT() < ekin[j+1] - (ekin[j+1] - ekin[j])/2.))
					{
						flux[j]++;
					}
				}
			}
		}
		else // Take care of the normalization if a particle did not reach the heliosphere in time
		{
			red++;
		}
	}

	for (unsigned int j = 0; j < ekin.size(); j++)
	{
		if (red != 0)
		{
			flux[j] = 1./(den.getTotalNumber()-red)*flux[j];
		}
		else
		{
			flux[j] = 1./den.getTotalNumber()*flux[j];
		}
	}
}

/**
 * @brief Destructor
 */
Distribution::~Distribution() {
}

/**
 * @brief Return flux at given energy
 *
 * @param i vector position of flux
 *
 * @return Flux value
 */
double Distribution::getFlux(int i) const {
	return flux[i];
}
