/*
 * import.h
 *
 *      Author: kappl
 */

#ifndef IMPORT_H_
#define IMPORT_H_

#include <iostream>
#include <map>
#include <vector>
#include <string>
#include <sstream>
#include <fstream>
#include "stringconst.h"
#include "constants.h"
#include "fileHelper.h"
#include "properties.h"

/**
 * @brief Import Handler to read input from files
 */
class Import
{
	private:
	std::map<std::string, std::string> controlData;
	std::map<std::string, int> allowedModels {{Stringconst::modelYamada, 1},
												{Stringconst::modelJokipii, 1},
												{Stringconst::modelPotgieter, 1},
												{Stringconst::modelBurger, 1},
												{Stringconst::modelStandard2D, 1},
												{Stringconst::modelCustom, 1}
												};
	std::map<int,double> angleData;
	std::map<std::pair<int, int>, double> nmData;
	/**
	* @brief Phi
	*/
	double phi;

	/**
	 * @brief Format of the output file
	 */
	std::string outputFormat;
	/**
	 * @brief Format of the input file
	 */
	std::string inputFormat;
	/**
	 * @brief Format of the output file
	 */
	std::string outputFileName;
	/**
	 * @brief Format of the input file
	 */
	std::string inputFileName;
	/**
	 * @brief Particle properties if provided by control.dat file
	 */
	Tproperties spec;
	/**
	 * @brief CR species which sould be modulated
	 */
	int modNumber;
	/**
	 * @brief spectral index for output rescaling
	 */
	double index;
	/**
	 * @brief Number of extra bins for quality improvement
	 */
	int extraBins;

	TimportStatus status;

	TmodelInformation modelInfo;

	void readProperties();
	int calculateCR(int, int, int);
	void calculateKappaZero(double);
	double averageAngle(int,int,int,int,int,int);
	double averageNeutronMonitorData(int, int, int, int);
	void parse(std::string, std::string, std::string);
	template <typename T> T convert(std::string);
	template <typename T> T check(std::string, T, std::string, std::string);

	public:
	Import(int, const char **);
	~Import();
	double getPhi() const;
	std::string getOutputFormat() const;
	std::string getInputFormat() const;
	std::string getOutputFileName() const;
	std::string getInputFileName() const;
	Tproperties getSpec() const;
	TmodelInformation getModelInformation() const;
	int getModNumber() const;
	int getExtraBins() const;
	double getIndex() const;
	TimportStatus getImportStatus() const;
};

#endif /* IMPORT_H_ */
