/*
 * main.cc
 *
 *      Author: kappl
 */

#include "spectrum.h"
#include "import.h"
#include <iostream>
#include <cmath>
#include <ctime>
#include <random>
#include <vector>
#include <chrono>

int main(int argc, const char **argv) {
	// Import data
	Import in(argc, argv);

	if (in.getImportStatus() == error)
	{
		std::cout << Stringconst::errorEnd << std::endl;
		return 0;
	}
	else if (in.getImportStatus() == version)
	{
		return 0;
	}

	// Check if local interstellar input is provided or not and build LIS spectrum
	Spectrum interstellar(in.getModelInformation(), in.getInputFileName(), in.getInputFormat(), in.getSpec());

	// Store original ekin data for reduceData
	std::vector<double> orig = interstellar.getEkin();

	// Add additional computed data points if necessary
	interstellar.quality(in.getExtraBins());

	// Generate TOA spectrum
	Spectrum toa(interstellar.getSpec(), interstellar.getEkin());

	// Simulate CR species
	toa.modulate(in.getModNumber()-1, in.getModelInformation(), interstellar);
	toa.reduceData(orig);
	// Determine output and write output to disk
	toa.output(in.getInputFormat(), in.getOutputFormat(), in.getInputFileName(),
				in.getOutputFileName(), in.getModNumber()-1, in.getIndex());

	// Force field approximation (if not given in control.dat phi = -1)
	if (in.getPhi() > 0)
	{
		Spectrum force(interstellar.getSpec(), interstellar.getEkin());
		force.forceField(in.getModNumber()-1, interstellar, in.getPhi());
		force.reduceData(orig);
		force.output(in.getInputFormat(), in.getOutputFormat(), in.getInputFileName(),
				in.getOutputFileName() + "Force", in.getModNumber()-1, in.getIndex());
	}

	// Write LIS flux to disk
	interstellar.reduceData(orig);
	interstellar.output(in.getInputFormat(), in.getOutputFormat(), in.getInputFileName(),
			in.getOutputFileName() + "LIS", in.getModNumber()-1, in.getIndex());

	std::cout << std::endl << Stringconst::infoSuccess << std::endl;
	return 0;
}
