/*
 * particle.h
 *
 *      Author: kappl
 */

#ifndef PARTICLE_H_
#define PARTICLE_H_

#include "Iparticle.h"

/**
 * @brief Implementation of a single particle
 */
class Particle : public IParticle
{
	private:
	void step(Random&);
	double getDeltaT() const;
	long getDeltaTime() const;

	protected:
	/**
	 * @brief radial coordinate
	 */
	double r;
	/**
	 * @brief angular (theta) coordinate
	 */
	double theta;
	/**
	 * @brief angular (phi) coordinate
	 */
	double phi;
	/**
	 * @brief time coordinate
	 */
	double time;
	/**
	 * @brief Particle properties
	 */
	Tproperties spec;
	/**
	 * @brief Data which are calculated in method void calculate()
	 */
	TstepData data;
	/**
	 * @brief dt from control file
	 */
	double dt;
	/**
	 * @brief Boundary of the heliosphere (model dependent)
	 */
	double heliosphereBoundary;
	/**
	 * @brief Coefficient between the parallel and perpendicular diffusion tensor
	 */
	double perPart;
	/**
	 * @brief Normalization of the diffusion tensor
	 */
	double kappa0;
	/**
	 * @brief Normalization of the solar magnetic field
	 */
	double BfieldScaling;
	/**
	 * @brief Solar magnetic field polarity
	 */
	int polarity;
	/**
	 * @brief HCS tilt angle
	 */
	double angle;

	virtual double getDeltaR(Random&) const = 0;
	double getDeltaTheta(Random&) const;
	double getDeltaPhi() const;

	void calculate();

	double getV() const;
	double getP2() const;
	double getRigidity() const;
	double getBeta() const;
	double getPolarity() const;
	double getAngle() const;
	double getTanPsi() const;
	double convertDeg(double) const;
	double getLarmorRadius() const;
	double getB0OverB() const;
	double getB() const;
	double getTheta() const;
	double getPhi() const;
	double getTime() const;
	double getMass() const;
	double getCharge() const;
	int getHeaviside() const;
	double getKappaPerpendicular() const;
	double getKappaThetaTheta() const;

	public:
	Particle(const TmodelInformation&, const Tproperties&);
	virtual ~Particle();

	long simulate(Random&);
	double getR() const;
	double getT() const;
	double getHeliosphereBoundary() const;
	double getISFlux(int) const;
	double getISFlux(bool) const;
	double getISFlux(bool, int) const;
};


#endif /* PARTICLE_H_ */
