/*
 * properties.h
 *
 *      Author: kappl
 */

#ifndef PROPERTIES_H_
#define PROPERTIES_H_

#include <string>

/**
 * @brief Structure for particle properties (charge, mass and kinetic energy T)
 */
typedef struct properties
{
	/**
	 * @brief Charge of a particle
	 */
	int charge;
	/**
	 * @brief Mass of a particle
	 */
	double mass;
	/**
	 * @brief Kinetic energy T of a particle
	 */
	double T;
} Tproperties;

/**
 * @brief Structure for model information determined in import.cc from control file
 */
typedef struct modelInformation {
	/**
	 * @brief Model name
	 */
	std::string model;
	/**
	 * @brief Scaling factor for the diffusion tensor
	 */
	double kappaScaling;
	/**
	 * @brief Scaling factor for the magnetic field
	 */
	double BfieldScaling;
	/**
	 * @brief Tilt angle
	 */
	double angle;
	/**
	 * @brief Solar magnetic field polarity
	 */
	int polarity;
	/**
	 * @brief Total number of particles
	 */
	int totalNumber;
	/**
	 * @brief Time step
	 */
	int dt;
} TmodelInformation;

/**
 * @brief Structure for quantities which are computed every step
 */
typedef struct stepData {
	/**
	 * @brief tan(psi)
	 */
	double tanPsi;
	/**
	 * @brief Particle velocity
	 */
	double beta;
	/**
	 * @brief Particle rigidity
	 */
	double rigidity;
	/**
	 * @brief Parallel component of the diffusion tensor
	 */
	double parallel;
	/**
	 * @brief Perpedndicular component of the diffusion tensor
	 */
	double perpendicular;
	/**
	 * @brief Angle for drift effects in models based on model ref4
	 */
	double alphaH;
} TstepData;

/**
 * @brief enum with different status informations for Solarprop
 */
typedef enum importStatus {
	success,
	error,
	version
} TimportStatus;

#endif /* PROPERTIES_H_ */
