/*
 * random.cc
 *
 *      Author: kappl
 */

#include "random.h"
#include <cstdlib>
#include <cmath>
#include <random>
#include <chrono>

/**
 * @brief Constructor
 *
 * @param g Random number generator
 */
Random::Random(std::default_random_engine g) {
	generator = g;
}

/**
 * @brief Constructor
 *
 * @param i Integer for random seed (e.g. thread number)
 */
Random::Random(int i) {
	generator.seed(i + std::chrono::system_clock().now().time_since_epoch()/std::chrono::microseconds(1));
}

/**
 * @brief Destructor
 */
Random::~Random() {
}

/**
 * @brief Simple Box-Muller routine to compute random number
 *
 * @return Gaussian random number
 */
double Random::getNumberBoxMuller() {
	double x = 0.;
	double y = 0.;
	double euclid = 0.;

	do
	{
	    x = 2.0 * rand() / static_cast<double>(RAND_MAX)-1;
	    y = 2.0 * rand() / static_cast<double>(RAND_MAX)-1;
	    euclid = x*x + y*y;
	}
	while (euclid >= 1.0);

	return x*sqrt(-2*log(euclid)/euclid);
}

/**
 * @brief Calls C++ random number generator
 *
 * @return Gaussian random number
 */
double Random::getNumber() {
	return normal_dist(generator);
}

/**
 * @brief Get uniform distributed value between 0 and pi
 *
 * @return Uniform distributed random number
 */
double Random::getRandZero2Pi() {
	std::uniform_real_distribution<double> uniform_dist(0, 2.*Constants::pi);

	return uniform_dist(generator);
}

