/*
 * spectrum.h
 *
 *      Author: kappl
 */

#ifndef SPECTRUM_H_
#define SPECTRUM_H_

#include <map>
#include <vector>
#include <string>
#include <CCfits/CCfits>
#include "density.h"
#include "distribution.h"
#include "fileHelper.h"

/**
 * @brief Class for handling distributions containing the spectra of CR
 */
class Spectrum
{
	private:
	std::vector<double> ekin;
	std::vector<std::vector<double>> flux;
	std::vector<Tproperties> spec;

	std::map<std::pair<int,int>,double> massDict = {{{1,1}, 0.938}, {{-1,1}, 0.938},
													{{1,0}, 0.000511}, {{-1,0}, 0.000511},
													{{1,2}, 2.*0.938}, {{-1,2}, 2.*0.938},
													{{1,3}, 3.*0.938}, {{-1,3}, 3.*0.938},
													{{2,3}, 3.*0.938}, {{-2,3}, 3.*0.938},
													{{2,4}, 4.*0.938}, {{-2,4}, 4.*0.938},
													{{3,6}, 6.*0.938}, {{3,7}, 7.*0.938},
													{{4,7}, 7.*0.938}, {{4,9}, 9.*0.938},
													{{4,10}, 10.*0.938}, {{5,10}, 10.*0.938},
													{{5,11}, 11.*0.938}, {{6,12}, 12.*0.938},
													{{6,13}, 13.*0.938}, {{6,14}, 14.*0.938},
													{{7,14}, 14.*0.938},
													{{7,15}, 15.*0.938}, {{8,16}, 16.*0.938},
													{{8,17}, 17.*0.938}, {{8,18}, 18.*0.938},
													{{9,19}, 19.*0.938}, {{10,20}, 20.*0.938},
													{{10,21}, 21.*0.938}, {{10,22}, 22.*0.938},
													{{11,22}, 22.*0.938},
													{{11,23}, 23.*0.938}, {{12,24}, 24.*0.938},
													{{12,25}, 25.*0.938}, {{12,26}, 26.*0.938},
													{{13,26}, 26.*0.938}, {{13,27}, 27.*0.938},
													{{14,28}, 28.*0.938}, {{14,29}, 29.*0.938},
													{{14,30}, 30.*0.938}, {{14,32}, 32.*0.938},
													{{15,31}, 31.*0.938},
													{{16,32}, 32.*0.938}, {{16,33}, 33.*0.938},
													{{16,34}, 34.*0.938}, {{16,36}, 36.*0.938},
													{{17,35}, 35.*0.938}, {{17,36}, 36.*0.938},
													{{17,37}, 37.*0.938}, {{18,36}, 36.*0.938},
													{{18,37}, 37.*0.938}, {{18,38}, 38.*0.938},
													{{18,40}, 40.*0.938}, {{19,39}, 39.*0.938},
													{{19,40}, 40.*0.938}, {{19,41}, 41.*0.938},
													{{20,40}, 40.*0.938}, {{20,41}, 41.*0.938},
													{{20,42}, 42.*0.938}, {{20,43}, 43.*0.938},
													{{20,44}, 44.*0.938}, {{20,46}, 46.*0.938},
													{{20,48}, 48.*0.938}, {{21,45}, 45.*0.938},
													{{22,44}, 44.*0.938}, {{22,46}, 46.*0.938},
													{{22,47}, 47.*0.938}, {{22,48}, 48.*0.938},
													{{22,49}, 49.*0.938}, {{22,50}, 50.*0.938},
													{{23,49}, 49.*0.938}, {{23,50}, 50.*0.938},
													{{23,51}, 51.*0.938}, {{24,50}, 50.*0.938},
													{{24,51}, 51.*0.938}, {{24,52}, 52.*0.938},
													{{24,53}, 53.*0.938}, {{24,54}, 54.*0.938},
													{{25,53}, 53.*0.938}, {{25,54}, 54.*0.938},
													{{25,55}, 55.*0.938}, {{26,54}, 54.*0.938},
													{{26,55}, 55.*0.938}, {{26,56}, 56.*0.938},
													{{26,57}, 57.*0.938}, {{26,58}, 58.*0.938},
													{{26,60}, 60.*0.938}, {{27,57}, 57.*0.938},
													{{27,59}, 59.*0.938}, {{28,56}, 56.*0.938},
													{{28,58}, 58.*0.938}, {{28,59}, 59.*0.938},
													{{28,60}, 60.*0.938}, {{28,61}, 61.*0.938},
													{{28,62}, 62.*0.938}, {{28,64}, 64.*0.938}
													};
	void readFITS(std::string);
	void readGALPROP(CCfits::FITS*);
	void readDRAGON(CCfits::FITS*);
	Tproperties checkSpecies(int, int);
	void readText(std::string, Tproperties);
	void rescaleWithP2(int);
	void modulate(const Spectrum&, const std::vector<Distribution>&, int);
	void modulate(const TmodelInformation&, const Spectrum&, int);
	void modulate(const TmodelInformation&, const Spectrum&);
	void forceField(const Spectrum&, double, int);
	void forceField(const Spectrum&, double);
	void output(std::string, std::string);
	void output(std::string, int i);
	void output(std::string);
	void rescaleFlux(double);

	double getP2(const Tproperties&) const;

	public:
	Spectrum(const std::vector<Tproperties>&, const std::vector<double>&);
	Spectrum(const TmodelInformation&, std::string, std::string, Tproperties);
	~Spectrum();

	void output(std::string, std::string, std::string, std::string, int, double);

	std::vector<double> getEkin() const;
	std::vector<Tproperties> getSpec() const;
	std::vector<double> getFlux(int) const;

	void quality(int);
	void reduceData(const std::vector<double>&);

	void forceField(int, const Spectrum&, double);

	void modulate(int, const TmodelInformation&, const Spectrum&);
};

#endif /* SPECTRUM_H_ */
